/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.mapper.rdb.adapter;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;

/**
 * This class is a sql adapter for the Firebird relational database.
 * @author P. Dechamboux
 */
public class FirebirdAdapter extends BasicRdbAdapter {

	public FirebirdAdapter() {
		super("firebird");
	}

	public String getSqlType(int typeCode, boolean usedInPK, int size, int scale) throws RdbAdapterException {
		String res;
		switch (typeCode) {
		case TYPECODE_BOOLEAN:
		case TYPECODE_OBJBOOLEAN:
			res = "char(1)";
			break;
		case TYPECODE_LONG:
		case TYPECODE_OBJLONG:
			res = "NUMERIC(18)";
			break;
		case TYPECODE_BYTE:
		case TYPECODE_OBJBYTE:
			return "SMALLINT";
		case TYPECODE_FLOAT:
		case TYPECODE_OBJFLOAT:
			return "FLOAT";
		case TYPECODE_DOUBLE:
		case TYPECODE_OBJDOUBLE:
			return "DOUBLE PRECISION";
		case TYPECODE_BYTEARRAY:
//        case TYPECODE_CHARARRAY:
		case TYPECODE_SERIALIZED:
			if (size != NOSIZE) {
				res = "BLOB(" + size + ")";
			}
			res = "BLOB";
			break;
		case TYPECODE_BIGINTEGER:
			if (size != NOSIZE) {
				res = "DECIMAL(" + size + ",0)";
			}
			return "DECIMAL(18,0)";
		case TYPECODE_BIGDECIMAL:
			if (size != NOSIZE) {
				res = "DECIMAL(" + size + ",4)";
			}
			return "DECIMAL(18,4)";
		default:
			res = super.getSqlType(typeCode, usedInPK, size, scale);
		}
		if (usedInPK) {
			res += " NOT NULL";
		}
		return res;
	}

	public boolean getBoolean(ResultSet rs, int idx, boolean nullValue) throws SQLException {
		String res = rs.getString(idx);
		return (rs.wasNull() || res == null ? nullValue : res.equals("T"));
	}

	public Boolean getOboolean(ResultSet rs, int idx, Boolean nullValue) throws SQLException {
		String res = rs.getString(idx);
		return (rs.wasNull() || res == null
				? nullValue
				: Boolean.valueOf(res.equals("T")));
	}

	public void setBoolean(PreparedStatement ps, int idx, boolean o)
			throws SQLException {
		ps.setString(idx, (o ? "T" : "F"));
	}

	public void setOboolean(PreparedStatement ps, int idx, Boolean o)
			throws SQLException {
		ps.setString(idx, (o.booleanValue() ? "T" : "F"));
	}

	public void setOchar(PreparedStatement ps, int idx, Character o)
			throws SQLException {
		ps.setString(idx, o.toString());
	}

	public void writeTableAlias(String alias, StringBuffer sb) {
		sb.append(" ");
		sb.append(alias);
	}
}
