/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.mapper.rdb.adapter.api;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.io.Serializable;
import java.io.IOException;

/**
 * Adapter for Resultset management
 *
 * @author S.Chassande-Barrioz
 */
public interface ResultsetAdapter {
	/**
	 * Get a boolean value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	boolean getBoolean(ResultSet rs, int idx, boolean nullValue) throws SQLException;

	/**
	 * Get a Boolean value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Boolean getOboolean(ResultSet rs, int idx, Boolean nullValue) throws SQLException;

	/**
	 * Get a char value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	char getChar(ResultSet rs, int idx, char nullValue) throws SQLException;

	/**
	 * Get a Character value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Character getOchar(ResultSet rs, int idx, Character nullValue) throws SQLException;

	/**
	 * Get a byte value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	byte getByte(ResultSet rs, int idx, byte nullValue) throws SQLException;

	/**
	 * Get a Byte value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Byte getObyte(ResultSet rs, int idx, Byte nullValue) throws SQLException;

	/**
	 * Get a short value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	short getShort(ResultSet rs, int idx, short nullValue) throws SQLException;

	/**
	 * Get a Short value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Short getOshort(ResultSet rs, int idx, Short nullValue) throws SQLException;

	/**
	 * Get a int value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	int getInt(ResultSet rs, int idx, int nullValue) throws SQLException;

	/**
	 * Get a Integer value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Integer getOint(ResultSet rs, int idx, Integer nullValue) throws SQLException;

	/**
	 * Get a long value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	long getLong(ResultSet rs, int idx, long nullValue) throws SQLException;

	/**
	 * Get a Long value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Long getOlong(ResultSet rs, int idx, Long nullValue) throws SQLException;

	/**
	 * Get a float value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	float getFloat(ResultSet rs, int idx, float nullValue) throws SQLException;

	/**
	 * Get a Float value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Float getOfloat(ResultSet rs, int idx, Float nullValue) throws SQLException;

	/**
	 * Get a double value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	double getDouble(ResultSet rs, int idx, double nullValue) throws SQLException;

	/**
	 * Get a Double value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Double getOdouble(ResultSet rs, int idx, Double nullValue) throws SQLException;

	/**
	 * Get a String value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	String getString(ResultSet rs, int idx, String nullValue) throws SQLException;

	/**
	 * Get a Date value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Date getDate(ResultSet rs, int idx, Date nullValue) throws SQLException;

	/**
	 * Get a Date value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Date getDate(ResultSet rs, int idx, String sqlColumnType, Date nullValue) throws SQLException;

	/**
	 * Get a BigDecimal value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	BigDecimal getBigDecimal(ResultSet rs, int idx, BigDecimal nullValue) throws SQLException;

	/**
	 * Get a BigInteger value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	BigInteger getBigInteger(ResultSet rs, int idx, BigInteger nullValue) throws SQLException;

	/**
	 * Get a char[] value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	char[] getCharArray(ResultSet rs, int idx, char[] nullValue) throws SQLException;

	/**
	 * Get a byte[] value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	byte[] getByteArray(ResultSet rs, int idx, byte[] nullValue) throws SQLException;

	/**
	 * Get a Serializable value from a ResultSet.
	 * @param rs is the result where to get the value
	 * @param idx is the index of the value in the resultset
	 * @param nullValue represents the nullvalue to return if the column is null
	 * in the resultset
	 */
	Serializable getSerialized(ResultSet rs, int idx, Serializable nullValue)
			throws SQLException, IOException, ClassNotFoundException;

}
